"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const exchangeUtils = require("../../../helpers/utils");
const uris = require("../../../helpers/uris");
const anypoint_cli_command_1 = require("anypoint-cli-command");
const _ = require('lodash');
class Upload extends anypoint_cli_command_1.AuthenticationBaseCommand {
    constructor() {
        super(...arguments);
        this.upload = async (flags, args) => {
            const asset = {
                organizationId: this.session.selectedOrganization.id,
                st: { ...exchangeUtils.assetIdentifierValidator(args.assetIdentifier, this.session.selectedOrganization.id) },
            };
            exchangeUtils.validateArgumentOptions(flags);
            const mainMetadata = _.pickBy({
                name: flags.name,
                description: flags.description,
                type: flags.type,
                dependencies: flags.dependencies,
                tags: flags.tags,
                keywords: flags.keywords,
                status: flags.status,
            }, _.identity);
            const { properties = {}, categories = {}, fields = {}, headers = {} } = flags;
            const files = this.buildFilesFromObject(flags.files);
            exchangeUtils.logProperties(this, asset, 'Uploading asset');
            exchangeUtils.logProperties(this, mainMetadata, 'Asset metadata');
            exchangeUtils.logProperties(this, properties, 'Asset properties');
            exchangeUtils.logProperties(this, categories, 'Asset categories');
            exchangeUtils.logProperties(this, fields, 'Asset fields');
            exchangeUtils.logProperties(this, files, 'Asset files');
            exchangeUtils.logProperties(this, headers, 'Request headers');
            await exchangeUtils.asyncUploadRequest(this, this.uploadFunction, asset, mainMetadata, properties, files, fields, categories, headers);
        };
        this.buildFilesFromObject = (filesArgument = {}) => {
            const files = [];
            for (const fileObjectKey of Object.keys(filesArgument)) {
                const fileObjectValue = filesArgument[fileObjectKey];
                if (typeof fileObjectValue === 'string') {
                    files.push({ packaging: fileObjectKey, path: fileObjectValue });
                }
                else if (typeof fileObjectValue === 'object') {
                    const classifier = fileObjectKey;
                    for (const packaging of Object.keys(fileObjectValue)) {
                        const filePath = fileObjectValue[packaging];
                        files.push({ classifier, packaging: packaging, path: filePath });
                    }
                }
            }
            return files;
        };
        this.uploadFunction = async (asset, formData, headers = {}) => {
            this.log('Sending request to exchange');
            const url = uris.postAsset(asset);
            const response = await this.req.post(url, formData, { headers: {
                    ...formData.getHeaders(headers),
                    'x-from-service': exchangeUtils.X_FROM_SERVICE_HEADER,
                    'x-from-service-version': 'v2',
                },
            });
            return response.data.publicationStatusLink;
        };
    }
    async run() {
        const { args, flags } = await this.parse(Upload);
        try {
            await this.upload(flags, args);
        }
        catch (error) {
            this.error(error);
        }
    }
}
exports.default = Upload;
Upload.description = 'Upload a rest-api, soap-api, http-api, custom, app, template, example, policy and connector asset. ' +
    'Basic example to upload a raml asset: exchange:asset:upload --name "Raml Asset" --description "RAML" --properties=\'{"mainFile":"encrypt.raml", "apiVersion":"v1"}\' --files=\'{"raml.zip":".../file-path/raml.zip" }"\' raml-asset/1.0.0. ' +
    'Describing the asset. ' + exchangeUtils.assetIdentifierFormatMsg + '\n' +
    'To learn more about the creation endpoint visit https://anypoint.mulesoft.com/exchange/portals/anypoint-platform/f1e97bc6-315a-4490-82a7-23abe036327a.anypoint-platform/exchange-experience-api/minor/2.0/pages/Asset%20Creation/';
Upload.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
    help: core_1.Flags.help({ char: 'h' }),
    name: core_1.Flags.string({ description: 'Asset name (required if no pom file is specified)' }),
    description: core_1.Flags.string({ description: 'Asset description' }),
    type: core_1.Flags.string({ description: 'Asset Type. (required if no file is uploaded. If it is uploaded, type will be inferred from classifier)' +
            'Supported Values: ' + exchangeUtils.supportedTypes.join(', ') }),
    keywords: core_1.Flags.string({ description: 'Keywords. Should be used as a comma separated list. E.g. "keyword1,keyword2,keyword3"' }),
    tags: core_1.Flags.string({ description: 'Tags. Should be used as a comma separated list. E.g. "tag1,tag2,tag3"' }),
    categories: core_1.Flags.string({ description: 'Categories. Should be used as --categories=\'{"someKey": "someValue"}\'', parse: JSON.parse }),
    fields: core_1.Flags.string({ description: 'Custom Fields. Should be used as --fields=\'{"someKey": "someValue"}\'', parse: JSON.parse }),
    dependencies: core_1.Flags.string({ description: 'Asset dependencies, separated by ",". E.g. dependencies groupId:assetId:version,groupId2...' }),
    properties: core_1.Flags.option({
        description: 'Asset properties. Should be used as --properties=\'{"mainFile":"main.raml", "apiVersion":"v1", "assetLink": "http://api.com"}\'',
        parse: JSON.parse
    }),
    files: core_1.Flags.option({
        description: 'Asset file. (it should be identified as classifier.packaging or just "packaging" as key of a json. E.g. --files=\'{"raml.zip":".../file-path/raml.zip" }"\' (to send multiple files, use the same option multiple times. Example: --files=\'{"raml.zip":".../file-path/raml.zip" , "pom.xml": "./pom.xml"}\'',
        parse: JSON.parse
    }),
    status: core_1.Flags.string({ description: 'Asset status can be one of: development or published', options: ['development', 'published'] }),
    headers: core_1.Flags.string({ description: 'Headers. Should be used as --headers=\'{"headerName": "headerValue"}\'' +
            'Supported Headers: ' + exchangeUtils.supportedHeaders.join(', ') }),
};
Upload.args = [{ name: 'assetIdentifier', required: true }, { name: 'properties' }];
